/*
 * FileUtil methods tests
 * Testing FileUtil methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>

#define TMP_FILE    "test.txt"

static FILE *fp = NULL;
static gint f_len = 0;

static int creat_tmpfile()
{
    fp = g_fopen(TMP_FILE, "w");
    if (fp == NULL)
        return -1;
    f_len = g_fprintf(fp, "%s", "Hello, LSB");
    fclose(fp);
    fp = NULL;
    return 0;
}

static int remove_tmpfile()
{
    if (fp != NULL){
        fclose(fp);
        fp = NULL;
    }
    return g_unlink(TMP_FILE);
}


/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "FileUtil.c", G_FILE_TEST_IS_REGULAR
 * Expected: TRUE  
 */
static void test_purpose_1()
{
    #define FILE_N    "FileUtil.c"
    #define TEST_F    G_FILE_TEST_IS_REGULAR
    #define RET       TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "FileUtil.c", G_FILE_TEST_IS_SYMLINK
 * Expected: FALSE  
 */
static void test_purpose_2()
{
    #define FILE_N    "FileUtil.c"
    #define TEST_F    G_FILE_TEST_IS_SYMLINK
    #define RET       FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "../FileUtil", G_FILE_TEST_IS_DIR
 * Expected: TRUE  
 */
static void test_purpose_3()
{
    #define FILE_N    "../FileUtil"
    #define TEST_F    G_FILE_TEST_IS_DIR
    #define RET       TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "FileUtil.c", G_FILE_TEST_EXISTS
 * Expected: TRUE  
 */
static void test_purpose_4()
{
    #define FILE_N    "FileUtil.c"
    #define TEST_F    G_FILE_TEST_EXISTS
    #define RET       TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "foo", G_FILE_TEST_IS_SYMLINK
 * Expected: TRUE  
 */
static void test_purpose_5()
{
    #define FILE_N    "foo"
    #define TEST_F    G_FILE_TEST_IS_SYMLINK
    #define RET       TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_test
 * Params: const gchar *filename, GFileTest test
 * Returns: gboolean
 * Assertion: Returns TRUE if any of the tests in the bitfield test are TRUE. For example, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR) will return TRUE if the file exists; the check whether it's a directory doesn't matter since the existence test is TRUE. With the current set of available tests, there's no point passing in more than one test at a time.
 *            filename : a filename to test in the GLib file name encoding
 *            test : bitfield of GFileTest flags
 *            Returns : whether a test was TRUE 
 * Params Values: "FileUtil", G_FILE_TEST_IS_EXECUTABLE
 * Expected: TRUE  
 */
static void test_purpose_6()
{
    #define FILE_N    "FileUtil"
    #define TEST_F    G_FILE_TEST_IS_EXECUTABLE
    #define RET       TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (symlink ("FileUtil", "foo") < 0) {
        TEST_FAIL ("Failed to create symbolic link", NULL);
        return;
    }
    
    if (g_file_test(FILE_N, TEST_F) != RET){
        TEST_FAIL("g_g_file_test: failed", NULL);
        return;
    }
    
    if (unlink ("foo") < 0) {
        TEST_FAIL ("Failed to remove symbolic link", NULL);
        return;
    }
    
    #undef FILE_N
    #undef TEST_F
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_dir_read_name
 * Params: GDir *dir
 * Returns: const gchar*
 * Assertion: Retrieves the name of the next entry in the directory. The '.' and '..' entries are omitted. On Windows, the returned name is in UTF-8. On Unix, it is in the on-disk encoding.
 *            dir : a GDir* created by g_dir_open()
 *            Returns : The entry's name or NULL if there are no more entries. The return value is owned by GLib and must not be modified or freed.
 * Params Values: <%0%>
 * Expected: The return is not NULL  
 */
static void test_purpose_7()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GDir *dir = g_dir_open("../FileUtil", 0, NULL);
    if (g_ascii_strcasecmp(g_dir_read_name(dir), "") == 0){
        TEST_FAIL("g_dir_read_name: failed", NULL);
    }
    g_dir_close(dir);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_dir_open
 * Params: const gchar *path, guint flags, GError **error
 * Returns: GDir*
 * Assertion: Opens a directory for reading. The names of the files in the directory can then be retrieved using g_dir_read_name().
 *            path : the path to the directory you are interested in. On Unix in the on-disk encoding. On Windows in UTF-8
 *            flags : Currently must be set to 0. Reserved for future use.
 *            error : return location for a GError, or NULL. If non-NULL, an error will be set if and only if g_dir_open_fails.
 *            Returns : a newly allocated GDir on success, NULL on failure. If non-NULL, you must free the result with g_dir_close() when you are finished with it. 
 * Params Values: "../FileUtil"
 * Expected: "../FileUtil"  
 */
static void test_purpose_8()
{
    #define PATH    "../FileUtil"
    #define RET     "../FileUtil"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GError *err = NULL;
    GDir *dir = NULL;
    
    dir = g_dir_open(PATH, 0, &err);
    if (RET == NULL){
        if (dir != NULL){
            TEST_FAIL("g_dir_open: should be NULL", NULL);
        }
    }
    else{
        if (dir == NULL){
            TEST_FAIL("g_dir_open: should not be NULL", NULL);
        }
        g_dir_close(dir);
    }
    #undef PATH
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_dir_open
 * Params: const gchar *path, guint flags, GError **error
 * Returns: GDir*
 * Assertion: Opens a directory for reading. The names of the files in the directory can then be retrieved using g_dir_read_name().
 *            path : the path to the directory you are interested in. On Unix in the on-disk encoding. On Windows in UTF-8
 *            flags : Currently must be set to 0. Reserved for future use.
 *            error : return location for a GError, or NULL. If non-NULL, an error will be set if and only if g_dir_open_fails.
 *            Returns : a newly allocated GDir on success, NULL on failure. If non-NULL, you must free the result with g_dir_close() when you are finished with it. 
 * Params Values: "../FileUtil_tmp"
 * Expected: NULL  
 */
static void test_purpose_9()
{
    #define PATH    "../FileUtil_tmp"
    #define RET     NULL
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GError *err = NULL;
    GDir *dir = NULL;
    
    dir = g_dir_open(PATH, 0, &err);
    if (RET == NULL){
        if (dir != NULL){
            TEST_FAIL("g_dir_open: should be NULL", NULL);
        }
    }
    else{
        if (dir == NULL){
            TEST_FAIL("g_dir_open: should not be NULL", NULL);
        }
        g_dir_close(dir);
    }
    #undef PATH
    #undef RET
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_get_contents
 * Params: const gchar *filename, gchar **contents, gsize *length, GError **error
 * Returns: gboolean
 * Assertion: Reads an entire file into allocated memory, with good error checking.
 * If the call was successful, it returns TRUE and sets contents to the file contents and length to the length of the file contents in bytes. The string stored in contents will be nul-terminated, so for text files you can pass NULL for the length argument. If the call was not successful, it returns FALSE and sets error. The error domain is G_FILE_ERROR. Possible error codes are those in the GFileError enumeration. In the error case, contents is set to NULL and length is set to zero.
 *            filename : name of a file to read contents from, in the GLib file name encoding
 *            contents : location to store an allocated string
 *            length : location to store length in bytes of the contents, or NULL
 *            error : return location for a GError, or NULL
 *            Returns : TRUE on success, FALSE if an error occurred 
 * Params Values: File name = TMP_FILE
 * Expected: content = "Hello, LSB", length = f_len, Error_code = NULL  
 */
static void test_purpose_10()
{
    #define FILE_N    TMP_FILE
    #define CONTENT   "Hello, LSB"
    #define LENGTH    f_len
    #define ERR_CODE  NULL
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    struct stat statbuf;
    struct passwd *pwent;
    gsize len;
    gchar *buf;
    GError *err = NULL;
    
    if (creat_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: create file %s failed", TMP_FILE);
    }
    
    if (stat (TMP_FILE, &statbuf) < 0) {
        TEST_FAIL ("stat error", NULL);
        return;
    }
    
    if (chmod (TMP_FILE, statbuf.st_mode | S_IROTH)) {
        TEST_FAIL ("chmod error", NULL);
        return;
    }
    
    pwent = getpwnam ("lsb-desktop");
    if (!pwent) {
        TEST_FAIL ("lsb-test-desktop package is not installed properly", NULL);
        return;
    }
    
    if (seteuid (pwent->pw_uid) < 0) {
        TEST_FAIL ("cannot change effective uid from root to lsb-desktop", NULL);
        return;
    }
    
    if (g_file_get_contents(FILE_N, &buf, &len, &err)){
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be %d, now is %d", LENGTH, len);
        }
        else if (g_ascii_strcasecmp(CONTENT, buf) != 0){
            TEST_FAIL("g_file_get_contents: contents should be %s, now is %s", CONTENT, buf);
        }
    }
    else{
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be 0, now is %d", len);
        }
        else if (buf != CONTENT){
            TEST_FAIL("g_file_get_contents: contents should be NULL, now is %s", buf);
        }
        else if (err->code != ERR_CODE){
            TEST_FAIL("g_file_get_contents: error code should be %d, now is %d and message is %s", ERR_CODE, err->code, err->message);
        }
    }
    
    if (seteuid (getuid ()) < 0) {
        TEST_FAIL ("cannot set effective uid back to root", NULL);
        return;
    }
    
    if (remove_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: remove file %s failed", TMP_FILE);
    }
    
    #undef FILE_N
    #undef CONTENT
    #undef LENGTH
    #undef ERR_CODE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_get_contents
 * Params: const gchar *filename, gchar **contents, gsize *length, GError **error
 * Returns: gboolean
 * Assertion: Reads an entire file into allocated memory, with good error checking.
 * If the call was successful, it returns TRUE and sets contents to the file contents and length to the length of the file contents in bytes. The string stored in contents will be nul-terminated, so for text files you can pass NULL for the length argument. If the call was not successful, it returns FALSE and sets error. The error domain is G_FILE_ERROR. Possible error codes are those in the GFileError enumeration. In the error case, contents is set to NULL and length is set to zero.
 *            filename : name of a file to read contents from, in the GLib file name encoding
 *            contents : location to store an allocated string
 *            length : location to store length in bytes of the contents, or NULL
 *            error : return location for a GError, or NULL
 *            Returns : TRUE on success, FALSE if an error occurred 
 * Params Values: File name = "../FileUtil"
 * Expected: content = NULL, length = 0, Error_code = G_FILE_ERROR_ISDIR  
 */
static void test_purpose_11()
{
    #define FILE_N    "../FileUtil"
    #define CONTENT   NULL
    #define LENGTH    0
    #define ERR_CODE  G_FILE_ERROR_ISDIR
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    struct stat statbuf;
    struct passwd *pwent;
    gsize len;
    gchar *buf;
    GError *err = NULL;
    
    if (creat_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: create file %s failed", TMP_FILE);
    }
    
    if (stat (TMP_FILE, &statbuf) < 0) {
        TEST_FAIL ("stat error", NULL);
        return;
    }
    
    if (chmod (TMP_FILE, statbuf.st_mode | S_IROTH)) {
        TEST_FAIL ("chmod error", NULL);
        return;
    }
    
    pwent = getpwnam ("lsb-desktop");
    if (!pwent) {
        TEST_FAIL ("lsb-test-desktop package is not installed properly", NULL);
        return;
    }
    
    if (seteuid (pwent->pw_uid) < 0) {
        TEST_FAIL ("cannot change effective uid from root to lsb-desktop", NULL);
        return;
    }
    
    if (g_file_get_contents(FILE_N, &buf, &len, &err)){
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be %d, now is %d", LENGTH, len);
        }
        else if (g_ascii_strcasecmp(CONTENT, buf) != 0){
            TEST_FAIL("g_file_get_contents: contents should be %s, now is %s", CONTENT, buf);
        }
    }
    else{
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be 0, now is %d", len);
        }
        else if (buf != CONTENT){
            TEST_FAIL("g_file_get_contents: contents should be NULL, now is %s", buf);
        }
        else if (err->code != ERR_CODE){
            TEST_FAIL("g_file_get_contents: error code should be %d, now is %d and message is %s", ERR_CODE, err->code, err->message);
        }
    }
    
    if (seteuid (getuid ()) < 0) {
        TEST_FAIL ("cannot set effective uid back to root", NULL);
        return;
    }
    
    if (remove_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: remove file %s failed", TMP_FILE);
    }
    
    #undef FILE_N
    #undef CONTENT
    #undef LENGTH
    #undef ERR_CODE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_get_contents
 * Params: const gchar *filename, gchar **contents, gsize *length, GError **error
 * Returns: gboolean
 * Assertion: Reads an entire file into allocated memory, with good error checking.
 * If the call was successful, it returns TRUE and sets contents to the file contents and length to the length of the file contents in bytes. The string stored in contents will be nul-terminated, so for text files you can pass NULL for the length argument. If the call was not successful, it returns FALSE and sets error. The error domain is G_FILE_ERROR. Possible error codes are those in the GFileError enumeration. In the error case, contents is set to NULL and length is set to zero.
 *            filename : name of a file to read contents from, in the GLib file name encoding
 *            contents : location to store an allocated string
 *            length : location to store length in bytes of the contents, or NULL
 *            error : return location for a GError, or NULL
 *            Returns : TRUE on success, FALSE if an error occurred 
 * Params Values: File name = "/etc/shadow"
 * Expected: content = NULL, length = 0, Error_code = G_FILE_ERROR_ACCES  
 */
static void test_purpose_12()
{
    #define FILE_N    "/etc/shadow"
    #define CONTENT   NULL
    #define LENGTH    0
    #define ERR_CODE  G_FILE_ERROR_ACCES
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    struct stat statbuf;
    struct passwd *pwent;
    gsize len;
    gchar *buf;
    GError *err = NULL;
    
    if (creat_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: create file %s failed", TMP_FILE);
    }
    
    if (stat (TMP_FILE, &statbuf) < 0) {
        TEST_FAIL ("stat error", NULL);
        return;
    }
    
    if (chmod (TMP_FILE, statbuf.st_mode | S_IROTH)) {
        TEST_FAIL ("chmod error", NULL);
        return;
    }
    
    pwent = getpwnam ("lsb-desktop");
    if (!pwent) {
        TEST_FAIL ("lsb-test-desktop package is not installed properly", NULL);
        return;
    }
    
    if (seteuid (pwent->pw_uid) < 0) {
        TEST_FAIL ("cannot change effective uid from root to lsb-desktop", NULL);
        return;
    }
    
    if (g_file_get_contents(FILE_N, &buf, &len, &err)){
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be %d, now is %d", LENGTH, len);
        }
        else if (g_ascii_strcasecmp(CONTENT, buf) != 0){
            TEST_FAIL("g_file_get_contents: contents should be %s, now is %s", CONTENT, buf);
        }
    }
    else{
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be 0, now is %d", len);
        }
        else if (buf != CONTENT){
            TEST_FAIL("g_file_get_contents: contents should be NULL, now is %s", buf);
        }
        else if (err->code != ERR_CODE){
            TEST_FAIL("g_file_get_contents: error code should be %d, now is %d and message is %s", ERR_CODE, err->code, err->message);
        }
    }
    
    if (seteuid (getuid ()) < 0) {
        TEST_FAIL ("cannot set effective uid back to root", NULL);
        return;
    }
    
    if (remove_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: remove file %s failed", TMP_FILE);
    }
    
    #undef FILE_N
    #undef CONTENT
    #undef LENGTH
    #undef ERR_CODE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_get_contents
 * Params: const gchar *filename, gchar **contents, gsize *length, GError **error
 * Returns: gboolean
 * Assertion: Reads an entire file into allocated memory, with good error checking.
 * If the call was successful, it returns TRUE and sets contents to the file contents and length to the length of the file contents in bytes. The string stored in contents will be nul-terminated, so for text files you can pass NULL for the length argument. If the call was not successful, it returns FALSE and sets error. The error domain is G_FILE_ERROR. Possible error codes are those in the GFileError enumeration. In the error case, contents is set to NULL and length is set to zero.
 *            filename : name of a file to read contents from, in the GLib file name encoding
 *            contents : location to store an allocated string
 *            length : location to store length in bytes of the contents, or NULL
 *            error : return location for a GError, or NULL
 *            Returns : TRUE on success, FALSE if an error occurred 
 * Params Values: File name = "xyz.tmp"
 * Expected: content = NULL, length = 0, Error_code = G_FILE_ERROR_NOENT  
 */
static void test_purpose_13()
{
    #define FILE_N    "xyz.tmp"
    #define CONTENT   NULL
    #define LENGTH    0
    #define ERR_CODE  G_FILE_ERROR_NOENT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    struct stat statbuf;
    struct passwd *pwent;
    gsize len;
    gchar *buf;
    GError *err = NULL;
    
    if (creat_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: create file %s failed", TMP_FILE);
    }
    
    if (stat (TMP_FILE, &statbuf) < 0) {
        TEST_FAIL ("stat error", NULL);
        return;
    }
    
    if (chmod (TMP_FILE, statbuf.st_mode | S_IROTH)) {
        TEST_FAIL ("chmod error", NULL);
        return;
    }
    
    pwent = getpwnam ("lsb-desktop");
    if (!pwent) {
        TEST_FAIL ("lsb-test-desktop package is not installed properly", NULL);
        return;
    }
    
    if (seteuid (pwent->pw_uid) < 0) {
        TEST_FAIL ("cannot change effective uid from root to lsb-desktop", NULL);
        return;
    }
    
    if (g_file_get_contents(FILE_N, &buf, &len, &err)){
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be %d, now is %d", LENGTH, len);
        }
        else if (g_ascii_strcasecmp(CONTENT, buf) != 0){
            TEST_FAIL("g_file_get_contents: contents should be %s, now is %s", CONTENT, buf);
        }
    }
    else{
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be 0, now is %d", len);
        }
        else if (buf != CONTENT){
            TEST_FAIL("g_file_get_contents: contents should be NULL, now is %s", buf);
        }
        else if (err->code != ERR_CODE){
            TEST_FAIL("g_file_get_contents: error code should be %d, now is %d and message is %s", ERR_CODE, err->code, err->message);
        }
    }
    
    if (seteuid (getuid ()) < 0) {
        TEST_FAIL ("cannot set effective uid back to root", NULL);
        return;
    }
    
    if (remove_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: remove file %s failed", TMP_FILE);
    }
    
    #undef FILE_N
    #undef CONTENT
    #undef LENGTH
    #undef ERR_CODE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_get_contents
 * Params: const gchar *filename, gchar **contents, gsize *length, GError **error
 * Returns: gboolean
 * Assertion: Reads an entire file into allocated memory, with good error checking.
 * If the call was successful, it returns TRUE and sets contents to the file contents and length to the length of the file contents in bytes. The string stored in contents will be nul-terminated, so for text files you can pass NULL for the length argument. If the call was not successful, it returns FALSE and sets error. The error domain is G_FILE_ERROR. Possible error codes are those in the GFileError enumeration. In the error case, contents is set to NULL and length is set to zero.
 *            filename : name of a file to read contents from, in the GLib file name encoding
 *            contents : location to store an allocated string
 *            length : location to store length in bytes of the contents, or NULL
 *            error : return location for a GError, or NULL
 *            Returns : TRUE on success, FALSE if an error occurred 
 * Params Values: File name = "xyz.tmp"
 * Expected: content = NULL, length = 0, Error_code = G_FILE_ERROR_NOENT  
 */
static void test_purpose_14()
{
    #define FILE_N    "xyz.tmp"
    #define CONTENT   NULL
    #define LENGTH    0
    #define ERR_CODE  G_FILE_ERROR_NOENT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    struct stat statbuf;
    struct passwd *pwent;
    gsize len;
    gchar *buf;
    GError *err = NULL;
    
    if (creat_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: create file %s failed", TMP_FILE);
    }
    
    if (stat (TMP_FILE, &statbuf) < 0) {
        TEST_FAIL ("stat error", NULL);
        return;
    }
    
    if (chmod (TMP_FILE, statbuf.st_mode | S_IROTH)) {
        TEST_FAIL ("chmod error", NULL);
        return;
    }
    
    pwent = getpwnam ("lsb-desktop");
    if (!pwent) {
        TEST_FAIL ("lsb-test-desktop package is not installed properly", NULL);
        return;
    }
    
    if (seteuid (pwent->pw_uid) < 0) {
        TEST_FAIL ("cannot change effective uid from root to lsb-desktop", NULL);
        return;
    }
    
    if (g_file_get_contents(FILE_N, &buf, &len, &err)){
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be %d, now is %d", LENGTH, len);
        }
        else if (g_ascii_strcasecmp(CONTENT, buf) != 0){
            TEST_FAIL("g_file_get_contents: contents should be %s, now is %s", CONTENT, buf);
        }
    }
    else{
        if (len != LENGTH){
            TEST_FAIL("g_file_get_contents: length should be 0, now is %d", len);
        }
        else if (buf != CONTENT){
            TEST_FAIL("g_file_get_contents: contents should be NULL, now is %s", buf);
        }
        else if (err->code != ERR_CODE){
            TEST_FAIL("g_file_get_contents: error code should be %d, now is %d and message is %s", ERR_CODE, err->code, err->message);
        }
    }
    
    if (seteuid (getuid ()) < 0) {
        TEST_FAIL ("cannot set effective uid back to root", NULL);
        return;
    }
    
    if (remove_tmpfile() != 0){
        TEST_FAIL("g_file_get_contents: remove file %s failed", TMP_FILE);
    }
    
    #undef FILE_N
    #undef CONTENT
    #undef LENGTH
    #undef ERR_CODE
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EEXIST
 * Expected: G_FILE_ERROR_EXIST  
 */
static void test_purpose_15()
{
    #define ERRNO    EEXIST
    #define FILE_NO  G_FILE_ERROR_EXIST
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EISDIR
 * Expected: G_FILE_ERROR_ISDIR  
 */
static void test_purpose_16()
{
    #define ERRNO    EISDIR
    #define FILE_NO  G_FILE_ERROR_ISDIR
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EACCES
 * Expected: G_FILE_ERROR_ACCES  
 */
static void test_purpose_17()
{
    #define ERRNO    EACCES
    #define FILE_NO  G_FILE_ERROR_ACCES
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENAMETOOLONG
 * Expected: G_FILE_ERROR_NAMETOOLONG  
 */
static void test_purpose_18()
{
    #define ERRNO    ENAMETOOLONG
    #define FILE_NO  G_FILE_ERROR_NAMETOOLONG
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENOENT
 * Expected: G_FILE_ERROR_NOENT  
 */
static void test_purpose_19()
{
    #define ERRNO    ENOENT
    #define FILE_NO  G_FILE_ERROR_NOENT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENOTDIR
 * Expected: G_FILE_ERROR_NOTDIR  
 */
static void test_purpose_20()
{
    #define ERRNO    ENOTDIR
    #define FILE_NO  G_FILE_ERROR_NOTDIR
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENXIO
 * Expected: G_FILE_ERROR_NXIO  
 */
static void test_purpose_21()
{
    #define ERRNO    ENXIO
    #define FILE_NO  G_FILE_ERROR_NXIO
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EROFS
 * Expected: G_FILE_ERROR_ROFS  
 */
static void test_purpose_22()
{
    #define ERRNO    EROFS
    #define FILE_NO  G_FILE_ERROR_ROFS
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ETXTBSY
 * Expected: G_FILE_ERROR_TXTBSY  
 */
static void test_purpose_23()
{
    #define ERRNO    ETXTBSY
    #define FILE_NO  G_FILE_ERROR_TXTBSY
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EFAULT
 * Expected: G_FILE_ERROR_FAULT  
 */
static void test_purpose_24()
{
    #define ERRNO    EFAULT
    #define FILE_NO  G_FILE_ERROR_FAULT
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ELOOP
 * Expected: G_FILE_ERROR_LOOP  
 */
static void test_purpose_25()
{
    #define ERRNO    ELOOP
    #define FILE_NO  G_FILE_ERROR_LOOP
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENOSPC
 * Expected: G_FILE_ERROR_NOSPC  
 */
static void test_purpose_26()
{
    #define ERRNO    ENOSPC
    #define FILE_NO  G_FILE_ERROR_NOSPC
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENOMEM
 * Expected: G_FILE_ERROR_NOMEM  
 */
static void test_purpose_27()
{
    #define ERRNO    ENOMEM
    #define FILE_NO  G_FILE_ERROR_NOMEM
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EMFILE
 * Expected: G_FILE_ERROR_MFILE  
 */
static void test_purpose_28()
{
    #define ERRNO    EMFILE
    #define FILE_NO  G_FILE_ERROR_MFILE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENFILE
 * Expected: G_FILE_ERROR_NFILE  
 */
static void test_purpose_29()
{
    #define ERRNO    ENFILE
    #define FILE_NO  G_FILE_ERROR_NFILE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EBADF
 * Expected: G_FILE_ERROR_BADF  
 */
static void test_purpose_30()
{
    #define ERRNO    EBADF
    #define FILE_NO  G_FILE_ERROR_BADF
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EINVAL
 * Expected: G_FILE_ERROR_INVAL  
 */
static void test_purpose_31()
{
    #define ERRNO    EINVAL
    #define FILE_NO  G_FILE_ERROR_INVAL
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EPIPE
 * Expected: G_FILE_ERROR_PIPE  
 */
static void test_purpose_32()
{
    #define ERRNO    EPIPE
    #define FILE_NO  G_FILE_ERROR_PIPE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EAGAIN
 * Expected: G_FILE_ERROR_AGAIN  
 */
static void test_purpose_33()
{
    #define ERRNO    EAGAIN
    #define FILE_NO  G_FILE_ERROR_AGAIN
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EINTR
 * Expected: G_FILE_ERROR_INTR  
 */
static void test_purpose_34()
{
    #define ERRNO    EINTR
    #define FILE_NO  G_FILE_ERROR_INTR
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EIO
 * Expected: G_FILE_ERROR_IO  
 */
static void test_purpose_35()
{
    #define ERRNO    EIO
    #define FILE_NO  G_FILE_ERROR_IO
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = EPERM
 * Expected: G_FILE_ERROR_PERM  
 */
static void test_purpose_36()
{
    #define ERRNO    EPERM
    #define FILE_NO  G_FILE_ERROR_PERM
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = ENOSYS
 * Expected: G_FILE_ERROR_NOSYS  
 */
static void test_purpose_37()
{
    #define ERRNO    ENOSYS
    #define FILE_NO  G_FILE_ERROR_NOSYS
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_error_from_errno
 * Params: gint err_no
 * Returns: GFileError
 * Assertion: Gets a GFileError constant based on the passed-in errno. For example, if you pass in EEXIST this function returns G_FILE_ERROR_EXIST. Unlike errno values, you can portably assume that all GFileError values will exist.
 * Normally a GFileError value goes into a GError returned from a function that manipulates files. So you would use g_file_error_from_errno() when constructing a GError.
 *            err_no : an "errno" value
 *            Returns : GFileError corresponding to the given errno 
 * Params Values: err_no = 1000
 * Expected: G_FILE_ERROR_FAILED  
 */
static void test_purpose_38()
{
    #define ERRNO    1000
    #define FILE_NO  G_FILE_ERROR_FAILED
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    if (g_file_error_from_errno(ERRNO) != FILE_NO){
        TEST_FAIL("g_file_error_from_errno: should be %d, now is %d", FILE_NO, g_file_error_from_errno(ERRNO));
    }
    
    #undef ERRNO
    #undef FILE_NO
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_mkstemp
 * Params: gchar *tmpl
 * Returns: gint
 * Assertion: Opens a temporary file. See the mkstemp() documentation on most UNIX-like systems. This is a portability wrapper, which simply calls mkstemp() on systems that have it, and implements it in GLib otherwise.
 * The parameter is a string that should match the rules for mkstemp(), i.e. end in "XXXXXX". The X string will be modified to form the name of a file that didn't exist. The string should be in the GLib file name encoding. Most importantly, on Windows it should be in UTF-8.
 *            tmpl : template filename
 *            Returns : A file handle (as from open()) to the file opened for reading and writing. The file is opened in binary mode on platforms where there is a difference. The file handle should be closed with close(). In case of errors, -1 is returned. 
 * Params Values: "FileUtil.XXXXXX"
 * Expected: File handle  
 */
static void test_purpose_39()
{
    #define FILE_N    "FileUtil.XXXXXX"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gint fd;
    gchar *file_n;
    file_n = g_strdup_printf("%s", FILE_N);
    fd = g_mkstemp(file_n);
    if (fd == -1){
        TEST_FAIL("g_mkstemp: failed", NULL);
    }
    close(fd);
    g_unlink(file_n);
    g_free(file_n);
    
    #undef FILE_N
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_file_open_tmp
 * Params: const gchar *tmpl, gchar **name_used, GError **error
 * Returns: gint
 * Assertion: Opens a file for writing in the preferred directory for temporary files (as returned by g_get_tmp_dir()).
 * tmpl should be a string in the GLib file name encoding ending with six 'X' characters, as the parameter to g_mkstemp() (or mkstemp()). However, unlike these functions, the template should only be a basename, no directory components are allowed. If template is NULL, a default template is used.
 * Note that in contrast to g_mkstemp() (and mkstemp()) tmpl is not modified, and might thus be a read-only literal string.
 * The actual name used is returned in name_used if non-NULL. This string should be freed with g_free() when not needed any longer. The returned name is in the GLib file name encoding.
 *            tmpl : Template for file name, as in g_mkstemp(), basename only
 *            name_used : location to store actual name used
 *            error : return location for a GError
 *            Returns : A file handle (as from open()) to the file opened for reading and writing. The file is opened in binary mode on platforms where there is a difference. The file handle should be closed with close(). In case of errors, -1 is returned and error will be set.
 * Params Values: "FileUtil"
 * Expected: File handle  
 */
static void test_purpose_40()
{
    #define FILE_N    "FileUtil"
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    gchar *name_used;
    GError *err = NULL;
    gchar *file_n;
    gint fd;
    
    file_n = g_strdup_printf("%s.XXXXXX", FILE_N);
    fd = g_file_open_tmp(file_n, &name_used, &err);
    if (name_used == NULL){
        TEST_FAIL("g_file_open_tmp: failed", NULL);
    }
    else if (err != NULL){
        TEST_FAIL("g_file_open_tmp: %s", err->message);
    }
    else{
        close(fd);
        g_free(file_n);
        g_unlink(name_used);
        g_free(name_used);
    }
    
    #undef FILE_N
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for FileUtil ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {test_purpose_6, 1},
    {test_purpose_7, 1},
    {test_purpose_8, 1},
    {test_purpose_9, 1},
    {test_purpose_10, 1},
    {test_purpose_11, 1},
    {test_purpose_12, 1},
    {test_purpose_13, 1},
    {test_purpose_14, 1},
    {test_purpose_15, 1},
    {test_purpose_16, 1},
    {test_purpose_17, 1},
    {test_purpose_18, 1},
    {test_purpose_19, 1},
    {test_purpose_20, 1},
    {test_purpose_21, 1},
    {test_purpose_22, 1},
    {test_purpose_23, 1},
    {test_purpose_24, 1},
    {test_purpose_25, 1},
    {test_purpose_26, 1},
    {test_purpose_27, 1},
    {test_purpose_28, 1},
    {test_purpose_29, 1},
    {test_purpose_30, 1},
    {test_purpose_31, 1},
    {test_purpose_32, 1},
    {test_purpose_33, 1},
    {test_purpose_34, 1},
    {test_purpose_35, 1},
    {test_purpose_36, 1},
    {test_purpose_37, 1},
    {test_purpose_38, 1},
    {test_purpose_39, 1},
    {test_purpose_40, 1},
    {NULL,0}
};
