/*
 * GIOChannel methods tests
 * Testing GIOChannel methods
 *
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tet_api.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fontconfig.h>
#define _XFT_NO_COMPAT
#define _XFTCOMPAT_H_
#include <X11/Xlib.h>
#include <X11/Xft/Xft.h>
#include <pango/pangoft2.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <gmodule.h>
#include <gdk/gdkkeysyms.h>
#include <errno.h>
#include <unistd.h>

#include "libtestgen.h"
#include "libstr.h"
#include "libfile.h"
#include "libthr.h"

#define TEST_FAIL(str, ...) {test_failed ();tet_printf("%s, line %d: ", __FILE__, __LINE__);tet_printf (str, __VA_ARGS__);}

/*
 * global variables
 */

int test_purp_passed_num = 0;
int all_test_purp_num = 0;
int double_bug = 0;
unsigned test_passed_flag = 1;

/*
 * function prototypes
 */
static char* get_not_created_msg (void);
static char* get_wrong_type_msg (void);
static void test_passed (void);
static void test_failed (void);
static void startup_func();
static void cleanup_func();


gchar* bug_inf = NULL;
#include <fcntl.h>
#include <sys/stat.h>

static GMainLoop *main_loop;
static gint fd;

static gboolean recv_message(GIOChannel  *channel, GIOCondition cond, gpointer data)
{
    if (cond != G_IO_IN){
        TEST_FAIL("g_io_channel_unix_new: g_io_add_watch failed", NULL);
    }
    if (g_io_channel_unix_get_fd(channel) != fd){
         TEST_FAIL("g_io_channel_unix_new: g_io_channel_unix_get_fd failed", NULL);
    }
    g_main_loop_quit (main_loop);
    return TRUE;
}


/*
 * Name: g_io_channel_write_chars
 * Params: GIOChannel *channel, const gchar *buf, gssize count, gsize *bytes_written, GError **error
 * Returns: GIOStatus
 * Assertion: On seekable channels with encodings other than NULL or UTF-8, generic mixing of reading and writing is not allowed. A call to g_io_channel_write_chars() may only be made on a channel from which data has been read in the cases described in the documentation for g_io_channel_set_encoding().
 *            channel :     a GIOChannel
 *            buf :   a buffer to write data from
 *            count : the size of the buffer. If -1, the buffer is taken to be a nul-terminated string.
 *            bytes_written : The number of bytes written. This can be nonzero even if the return value is not G_IO_STATUS_NORMAL. If the return value is G_IO_STATUS_NORMAL and the channel is blocking, this will always be equal to count if count >= 0.
 *            error : A location to return an error of type GConvertError or GIOChannelError
 *            Returns :   the status of the operation. 
 * Params Values: GIOChannel *channel, const gchar *buf, gssize count, gsize *bytes_written, GError **error
 * Expected: GIOStatus  
 */
static void test_purpose_1()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GIOChannel *gio_w, *gio_r;
    GError *gio_err = NULL;
    gsize bytes_written;
    gsize bytes_read;
    GIOStatus status;
    gsize term_pos;
    gchar *str_return = NULL;
    
    GString *buf_write = g_string_new("Hello, LSB");
    GString *buf_read = g_string_sized_new(10);
    
    gio_w = g_io_channel_new_file("test.file", "w", &gio_err);
    if (gio_err){
        g_warning("Unable to open file %s: %s", "test.file", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_new_file failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_write_chars(gio_w, buf_write->str, buf_write->len, &bytes_written, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_write_chars: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    do
        status = g_io_channel_flush(gio_w, &gio_err);
    while (status == G_IO_STATUS_AGAIN);
    if (status == G_IO_STATUS_ERROR){
        g_warning(gio_err->message);
        g_error_free(gio_err);
        gio_err = NULL;
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_flush failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    gio_r = g_io_channel_new_file("test.file", "r", &gio_err);
    if (gio_err){
        g_warning("Unable to open file %s: %s", "test.file", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_new_file failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_read_chars(gio_r, buf_read->str, buf_write->len, &bytes_read, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_write_chars: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    if (g_ascii_strncasecmp(buf_read->str, buf_write->str, buf_write->len) != 0){
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_read_chars failed %s", buf_read->str);
    }
    
    status = g_io_channel_read_line(gio_r, &str_return, &bytes_read, &term_pos, &gio_err);
    if (gio_err){
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_read_line", NULL);
        g_error_free(gio_err);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    if(g_ascii_strcasecmp(str_return, "Hello, LSB\n") != 0){
        TEST_FAIL("g_io_channel_write_chars: g_io_channel_read_line failed %s", str_return);
    }
    
    g_clear_error(&gio_err);
    g_string_free(buf_write, TRUE);
    g_string_free(buf_read, TRUE);
    g_free(str_return);
    
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_io_channel_unix_new
 * Params: int fd
 * Returns: GIOChannel*
 * Assertion: Creates a new GIOChannel given a file descriptor. On UNIX systems this works for plain files, pipes, and sockets.
 * The returned GIOChannel has a reference count of 1.
 * The default encoding for GIOChannel is UTF-8. If your application is reading output from a command using via pipe, you may need to set the encoding to the encoding of the current locale (see g_get_charset()) with the g_io_channel_set_encoding() function.
 * If you want to read raw binary data without interpretation, then call the g_io_channel_set_encoding() function with NULL for the encoding argument.
 *            fd : a file descriptor.
 *            Returns : a new GIOChannel.
 * Params Values: int fd
 * Expected: GIOChannel*
 * 
 * Name: g_io_add_watch
 * Params: GIOChannel *channel, GIOCondition condition, GIOFunc func, gpointer user_data
 * Returns: guint
 * Assertion: Adds the GIOChannel into the main event loop with the default priority.
 *            channel : a GIOChannel.
 *            condition : the condition to watch for.
 *            func : the function to call when the condition is satisfied.
 *            user_data : user data to pass to func.
 *            Returns : the event source id.
 * Params Values: GIOChannel *channel, GIOCondition condition, GIOFunc func, gpointer user_data
 * Expected: guint
 * 
 * Name: g_io_channel_unix_get_fd
 * Params: GIOChannel *channel
 * Returns: gint
 * Assertion: Returns the file descriptor of the UNIX GIOChannel.
 *            channel : a GIOChannel, created with g_io_channel_unix_new().
 *            Returns : the file descriptor of the GIOChannel.
 * Params Values: GIOChannel *channel
 * Expected: gint  
 */
static void test_purpose_2()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GIOChannel *gio;
    guint id;
    
    fd = open("GIOChannel.c", O_RDONLY, 0);
    if (fd < 0){
        TEST_FAIL("g_io_channel_unix_new: fail to open GIOChannel.c", NULL);
        return;
    }
    gio = g_io_channel_unix_new(fd);
    if (gio == NULL){
        TEST_FAIL("g_io_channel_unix_new: fail to new fd", NULL);
        return;
    }
    id = g_io_add_watch(gio, G_IO_IN, recv_message, &id);
    main_loop =  g_main_loop_new (NULL, FALSE);
    g_main_loop_run(main_loop);
    g_main_loop_unref(main_loop);
    close(fd);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_io_add_watch_full
 * Params: GIOChannel *channel, gint priority, GIOCondition condition, GIOFunc func, gpointer user_data, GDestroyNotify notify
 * Returns: guint
 * Assertion: Adds the GIOChannel into the main event loop with the given priority.
 *            channel : a GIOChannel.
 *            priority : the priority of the GIOChannel source.
 *            condition : the condition to watch for.
 *            func : the function to call when the condition is satisfied.
 *            user_data : user data to pass to func.
 *            notify : the function to call when the source is removed.
 *            Returns : the event source id. 
 * Params Values: GIOChannel *channel, gint priority, GIOCondition condition, GIOFunc func, gpointer user_data, GDestroyNotify notif
 * Expected: guint  
 */
static void test_purpose_3()
{
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GIOChannel *gio;
    guint id;
    
    fd = open("GIOChannel.c", O_RDONLY, 0);
    if (fd < 0){
        TEST_FAIL("g_io_channel_unix_new: fail to open GIOChannel.c", NULL);
        return;
    }
    gio = g_io_channel_unix_new(fd);
    if (gio == NULL){
        TEST_FAIL("g_io_channel_unix_new: fail to new fd", NULL);
        return;
    }
    id = g_io_add_watch_full(gio, G_PRIORITY_DEFAULT, G_IO_IN, recv_message, &id, NULL);
    main_loop =  g_main_loop_new (NULL, FALSE);
    g_main_loop_run(main_loop);
    g_main_loop_unref(main_loop);
    close(fd);
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_io_channel_shutdown
 * Params: GIOChannel *channel, gboolean flush, GError **err
 * Returns: GIOStatus
 * Assertion: Close an IO channel. Any pending data to be written will be flushed if flush is TRUE. The channel will not be freed until the last reference is dropped using g_io_channel_unref().
 *            channel : a GIOChannel
 *            flush : if TRUE, flush pending
 *            err : location to store a GIOChannelError
 *            Returns : the status of the operation.
 * Params Values: GIOChannel *channel, gboolean flush, GError **err
 * Expected: GIOStatus  
 */
static void test_purpose_4()
{
    #define FLUSH   TRUE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GIOChannel *gio_w, *gio_r;
    GError *gio_err = NULL;
    gsize bytes_written;
    gsize bytes_read;
    GIOStatus status;
    gsize term_pos;
    gchar *str_return = NULL;
    
    GString *buf_write = g_string_new("Hello, LSB");
    GString *buf_read = g_string_sized_new(10);
    
    gio_w = g_io_channel_new_file("test.file", "w", &gio_err);
    if (gio_err){
        g_warning("Unable to open file %s: %s", "test.file", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: g_io_channel_new_file failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_write_chars(gio_w, buf_write->str, buf_write->len, &bytes_written, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_shutdown(gio_w, FLUSH, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    {
        struct stat stat_size;
        if (g_stat("test.file", &stat_size) != 0){
            TEST_FAIL("g_io_channel_shutdown: stat file failed", NULL);
        }
        else{
            if (FLUSH){
                if (stat_size.st_size != buf_write->len){
                     TEST_FAIL("g_io_channel_shutdown: not flush", NULL);
                }
            }
            else{
                if (stat_size.st_size != 0){
                    TEST_FAIL("g_io_channel_shutdown: flushed", NULL);
                }
            }
        }
    }
    
    #undef FLUSH
    
	
    test_passed ();

    double_bug = 0;
}

/*
 * Name: g_io_channel_shutdown
 * Params: GIOChannel *channel, gboolean flush, GError **err
 * Returns: GIOStatus
 * Assertion: Close an IO channel. Any pending data to be written will be flushed if flush is TRUE. The channel will not be freed until the last reference is dropped using g_io_channel_unref().
 *            channel : a GIOChannel
 *            flush : if TRUE, flush pending
 *            err : location to store a GIOChannelError
 *            Returns : the status of the operation.
 * Params Values: GIOChannel *channel, gboolean flush, GError **err
 * Expected: GIOStatus  
 */
static void test_purpose_5()
{
    #define FLUSH   FALSE
    

    bug_inf = NULL;
    test_passed_flag = 1;          
    all_test_purp_num++;
    fprintf (stderr, mark_symbol (all_test_purp_num));
    tet_printf("test case: %s, TP number: %d ", tet_pname, tet_thistest);

    GIOChannel *gio_w, *gio_r;
    GError *gio_err = NULL;
    gsize bytes_written;
    gsize bytes_read;
    GIOStatus status;
    gsize term_pos;
    gchar *str_return = NULL;
    
    GString *buf_write = g_string_new("Hello, LSB");
    GString *buf_read = g_string_sized_new(10);
    
    gio_w = g_io_channel_new_file("test.file", "w", &gio_err);
    if (gio_err){
        g_warning("Unable to open file %s: %s", "test.file", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: g_io_channel_new_file failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_write_chars(gio_w, buf_write->str, buf_write->len, &bytes_written, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    
    status = g_io_channel_shutdown(gio_w, FLUSH, &gio_err);
    if (gio_err){
        g_warning("Write chars error %s", gio_err->message);
        g_error_free(gio_err);
        TEST_FAIL("g_io_channel_shutdown: failed", NULL);
        g_string_free(buf_write, TRUE);
        g_string_free(buf_read, TRUE);
        return;
    }
    {
        struct stat stat_size;
        if (g_stat("test.file", &stat_size) != 0){
            TEST_FAIL("g_io_channel_shutdown: stat file failed", NULL);
        }
        else{
            if (FLUSH){
                if (stat_size.st_size != buf_write->len){
                     TEST_FAIL("g_io_channel_shutdown: not flush", NULL);
                }
            }
            else{
                if (stat_size.st_size != 0){
                    TEST_FAIL("g_io_channel_shutdown: flushed", NULL);
                }
            }
        }
    }
    
    #undef FLUSH
    
	
    test_passed ();

    double_bug = 0;
}


static 
char* get_not_created_msg (void)
{
        return "Widget is not created\n";
}

static 
char* get_wrong_type_msg (void)
{
        return "Created widget is of wrong type\n";
}


/*
 *  test_passed
 */
static void 
test_passed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    if (test_passed_flag)
    {
        test_purp_passed_num++;
    }
    tet_result (TET_PASS);
}

/*
 *  test_failed
 */
static void
test_failed (void)
{
    if (bug_inf && !double_bug)
    {
	double_bug = 1;
    }

    test_passed_flag = 0;
    tet_result (TET_FAIL);
}

/*
 * startup_func
 * open display and initialize other per test case things
 */
static void 
startup_func()
{
    int argc = 1;
    char** argv = NULL;

    argv = calloc(argc,  sizeof(char*));
    argv[0] = (char*)strdup(""); 
    gtk_init(&argc, &argv);
    fprintf (stderr, "Function tests for GIOChannel ");
}
 
/*
 * cleanup_func
 */
static void 
cleanup_func()
{

    fprintf (stderr, " %d passed", test_purp_passed_num);
    if (all_test_purp_num != test_purp_passed_num)
    {
        fprintf (stderr, " %d FAILED", all_test_purp_num - test_purp_passed_num);        
    }
    fprintf (stderr, "\n"); 
}


/* 
 *Program hooks into the tet harness 
 */
void (*tet_startup)() = startup_func;
void (*tet_cleanup)() = cleanup_func;

/*
 * TET Tests included in this file. 
 */
struct tet_testlist tet_testlist[] = {
    {test_purpose_1, 1},
    {test_purpose_2, 1},
    {test_purpose_3, 1},
    {test_purpose_4, 1},
    {test_purpose_5, 1},
    {NULL,0}
};
